@@ -1,7 +1,7 @@
 package builder::MyBuilder;
 use strict;
 use warnings;
-use parent qw(Module::Build);
+use base qw(Module::Build);
 
 sub new {
     my ( $self, %args ) = @_;
